/*-----------------------------------------------------------------------------
  name:         thing_hot_tub_button
  description:  hot tub temperature controller
  Version:      1
  By:           Tim Mikkelsen
  Created:      5/22/2016
  Updated:      5/26/2016

  This SparkFun Thing code/project is to control a hot tub via WiFi
  showing current hot tub temperature and being able to remotely
  set the hot tub to either minimum or maximum temperature.

  Hardware:
  - SparkFun ESP8266 Thing Dev Board          (Sparkfun WRL-13711)
  - TMP36 temperature sensor                  (Sparkfun SEN-10988)
  - 2 SPDT 5V Sealed relays                   (Sparkfun COM-00100)
  - 2x P2N2222A NPN transistors (BC337)       (Sparkfun COM-13689)
  - 2x 1N4001 1A/50V small signal diodes      (Sparkfun COM-08589)
  - 2x 330ohm 1/6 watt resistors              (Sparkfun COM-08377)
  - USB wall power supply or equivalent       (Sparkfun TOL-12890)

  Additional libraries and code segments:
  - base wifi server code (by dlf modified by Tom Igoe)
  - server button code (by zoomkat)

  To make the device work outside of your local network, enable
  your router to port forwarding using your external WAN address with
  the ThingPort value.  Setting the router varies by brand and will
  require looking at the router information.  Once set, an example is:
  - for a   local address   of   192.168.0.99
  - for a   ThingPort       of   40
  - for a   WAN address     of   50.170.100.101
  -----------------------------------------------------------
  - you would read status   at   http://50.170.100.101:40
  - you would set max       at   http://50.170.100.101:40/max
  - you would set min       at   http://50.170.100.101:40/min

*/

// libraries  -----------------------------------------------------------------
#include <SPI.h>
#include <ESP8266WiFi.h>

// constants  -----------------------------------------------------------------
const int AnalogPin = 0;              // note that the Thing ADC is 0-1V input
const int LedPin = 5;
const int RelayUpPin = 12;
const int RelayDownPin = 13;
const int DigitalPin = 16;

// variables  -----------------------------------------------------------------
float voltage;
int degreesF, degreesC;
int AnalogData;
bool DigitalData;
String WebRequest;
int ValRequest;


// WiFi and internet  ---------------------------------------------------------
char ssid[] = "SSID";                 // your network SSID (name)
char pass[] = "password";             // your network password
int keyIndex = 1234567890;            // your network key Index/password (for WEP)
IPAddress myIP(192, 168, 0, 99);      // your desired IP address
IPAddress gateway(192, 168, 0, 1);    // your network gateway address
IPAddress subnet(255, 255, 255, 0);   // subnet mask
int status = WL_IDLE_STATUS;
int const ThingPort = 40;             // port number to allow remote access
// be sure to go to
// http://mobinnet.ir/uploaded/common_ports.pdf
// or other list and pick an appropriate or
// unused port

// create devices  ------------------------------------------------------------
WiFiServer server(ThingPort);


// Arduino setup  -------------------------------------------------------------
void setup() {
  // initialize I/O  ----------------------------------------------------------
  pinMode(DigitalPin, INPUT_PULLUP);
  pinMode(LedPin, OUTPUT);
  pinMode(RelayUpPin, OUTPUT);
  pinMode(RelayDownPin, OUTPUT);

  // initialize serial and wait for port to open:  ----------------------------
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  // check for the presence of WiFi  ------------------------------------------
  if (WiFi.status() == WL_NO_SHIELD) {
    Serial.println("WiFi: Hardware not present");
    // don't continue:
    while (true);
  }

  // make sure Wifi is not an access point
  WiFi.mode(WIFI_STA);

  // attempt to connect to Wifi network  --------------------------------------
  while ( status != WL_CONNECTED) {
    Serial.print("WiFi: Attempting to connect to SSID: ");
    Serial.println(ssid);
    WiFi.config(myIP, gateway, subnet);
    status = WiFi.begin(ssid, pass, keyIndex);  // connect via WEP
    delay(10000);                               // wait 10 sec for connection
  }
  server.begin();                               // start web server
  printWifiStatus();                            // print connection status
}


// Arduino main loop  ---------------------------------------------------------
void loop() {
  WiFiClient client = server.available();       // listen for incoming clients
  if (client) {
    Serial.println("WiFi: new client");
    String WebRequest = client.readStringUntil('\r');    // Read the request
    Serial.println("Wifi: WebRequest is '" + WebRequest + "'");
    client.flush();

    // send a standard http response header  --------------------------------
    client.println("HTTP/1.1 200 OK");
    client.println("Content-Type: text/html");
    client.println();
    client.println("<HTML>");
    client.println("<HEAD>");
    client.println("<TITLE>Hot Tub Control</TITLE>");
    client.println("</HEAD>");

    client.println("<H1>Hot Tub Control</H1>");
    client.println();
    //   client.println("<!DOCTYPE HTML>");
    client.println("<html>");

    // output the value of A0 and the converted temperature  ----------------

    voltage = analogRead(AnalogPin) / 1023.0;
    degreesC = (voltage - 0.5) * 100.0;
    degreesF = ((voltage - 0.5) * 100.0) * (9.0 / 5.0) + 32.0;
    client.print("Hot Tub Temperature is ");
    client.print(degreesF);
    client.print("&deg;F  or ");
    client.print(degreesC);
    client.print("&deg;C");
    client.println("<br />");
    client.println("<br />");

    // mousedown buttons
    client.println("<input type=button background-color:green value=REFRESH onmousedown=location.href='/?ref;'>");
    client.println("<input type=button background-color:blue  value=MIN onmousedown=location.href='/?min;'>");
    client.println("<input type=button background-color:red   value=MAX onmousedown=location.href='/?max;'>");
    client.println("<br />");
    client.println("<br />");

    /*
      // display the value of D16 and set the LED to the same  ----------------
      DigitalData = digitalRead(DigitalPin);
      digitalWrite(LedPin, DigitalData);
      if (DigitalData) digitalWrite(LedPin, LOW); else digitalWrite(LedPin, HIGH);
      client.print("digital input is ");
      if (DigitalData) client.print("HIGH"); else client.print("LOW ");
      client.println("<br />");
    */

    // MIN temperature request  -----------------------------------------------
    if (WebRequest.indexOf("min") != -1) {
      Serial.print("Control: Setting thermostat to MINIMUM: ");
      for (int i = 0; i < 24 ; i++ ) {
        Serial.print(i);  Serial.print(" ");
        digitalWrite(RelayDownPin, HIGH);
        delay(200);
        digitalWrite(RelayDownPin, LOW);
        delay(100);
      }
      Serial.println(" completed");
      client.println("Hot Tub Temperature set to minimum");
      WebRequest = "";
    }

    // MIN temperature request  -----------------------------------------------
    else if (WebRequest.indexOf("max") != -1) {
      Serial.print("Control: Setting thermostat to MAXIMUM: ");
      for (int i = 0; i < 24 ; i++ ) {
        Serial.print(i);  Serial.print(" ");
        digitalWrite(RelayUpPin, HIGH);
        delay(200);
        digitalWrite(RelayUpPin, LOW);
        delay(100);
      }
      Serial.println(" completed");
      client.println("Hot Tub Temperature set to maximum");
      WebRequest = "";
    }

    client.println("</BODY>");
    client.println("</HTML>");
    delay(1);                   // give the web browser time to receive the data
    client.flush();             // close the connection
    client.stop();
    Serial.println("WiFi: client disonnected");
  }
}


void printWifiStatus() {
  Serial.print("WiFi: Status  SSID: ");        // print the SSID of the network you're attached to:
  Serial.print(WiFi.SSID());
  IPAddress ip = WiFi.localIP(); // print your WiFi shield's IP address:
  Serial.print("  IP Address: ");
  Serial.print(ip);
  long rssi = WiFi.RSSI();        // print the received signal strength:
  Serial.print("  signal strength (RSSI): ");
  Serial.print(rssi);
  Serial.println(" dBm");
}
